
/*  This file is part of vobcopy.
 *
 *  vobcopy is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  vobcopy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with vobcopy; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */


#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/stat.h>
#include "vobcopy.h"
/*for solaris, if we need to include some cdrom related stuff
#include <sys/cdio.h>
although I can't think what <linux/cdrom.h> was included for... */

/* included from cdtitle.c (thx nils *g)
 * get the Title
 * returns 0 on success or a value < 0 on error
 */
int get_cdromtitle(const char *device, char *title)
{
  int  filehandle = 0;
  int  i = 0, last = 0;
  
  /* open the device */
  if ( !(filehandle = open(device, O_RDONLY)) )
  {
      /* open failed */
      fprintf(stderr, "something wrong in cdtitle - please specify path as /dev/dvd or /dev/hdb\n");
      fprintf(stderr, "opening of the device failed\n");
      return -1;
  }
  
  /* seek to title of first track, which is at (track_no * 32768) + 40 */
  if ( 32808 != lseek(filehandle, 32808, SEEK_SET) ) 
  {
      /* seek failed */
      close(filehandle);
      fprintf(stderr, "something wrong in cdtitle - please specify path as /dev/dvd or /dev/hdb\n");
      fprintf(stderr, "couldn't seek into the drive\n");
      return -1;
  }
  
  /* read title */
  if ( 32 != read(filehandle, title, 32)) 
  {
      close(filehandle);
      fprintf(stderr, "something wrong in cdtitle - please specify path as /dev/dvd or /dev/hdb\n");
      fprintf(stderr, "couldn't read some bytes\n");
      return -1;
  }
  
  /* make sure string is terminated */
  title[32] = '\0';
  
  /* we don't need trailing spaces           */
  /* find the last character that is not ' ' */
  last = 32; /* default if string is 32 chars long */
  for ( i = 0; i < 32; i++ ) 
  {
      if ( title[i] != ' ' ) { last = i; }
  }
  
  title[last + 1] = '\0';
  return 0;
}


/* function to get the device of a given pathname */
/* returns 0 if successful and NOT mounted        */
/*         1 if successful and mounted            */
/* returns <0 if error                            */
int get_device( char *path, char *device )
{
  FILE	*tmp_streamin;
  char	tmp_bufferin[MAX_STRING];
  char  *pointer, tmp_path[256];
  int   k = 0;
  bool  mounted = FALSE;


  /* the string should have no trailing / */
  if( path[(strlen(path)-1)] == '/')
  {
      path[(strlen(path)-1)] = '\0';
  }

  /* remove video_ts if given */
  if( (pointer = strstr(path, "/video_ts")) || (pointer = strstr(path, "/VIDEO_TS")) )
  {
      *pointer='\0';
  }

  /* check if it is given as /dev/xxx already */
  if(strstr(path, "/dev/"))
  {
      strcpy( device, path);
  }
  else
  {

    /*
     *look through /etc/mtab to see if it's actually mounted
     */
     
    if( (tmp_streamin = fopen("/etc/mtab","r")))
      {
	strcpy(tmp_path, path);
	strcat(tmp_path, " "); /* otherwise it would detect that e.g. 
				  /cdrom is mounted even if only/cdrom1 is 
				  mounted */ 

	memset( tmp_bufferin, 0, MAX_STRING * sizeof(char));
	while( fgets( tmp_bufferin, MAX_STRING, tmp_streamin )) 
          {
	    if(strstr( tmp_bufferin, tmp_path))
              {
		mounted = TRUE;
	      }
	  }
	fclose(tmp_streamin);
      }
    else
      {
	fprintf(stderr, "Could not read /etc/mtab!");
	return -1;
      }  
    
    /*
     *read the device out of /etc/fstab 
     */

    if( (tmp_streamin = fopen("/etc/fstab","r")))
      {
	strcpy(tmp_path, path);

	memset( tmp_bufferin, 0, MAX_STRING * sizeof(char));
	while( fgets( tmp_bufferin, MAX_STRING, tmp_streamin )) 
	  {
	    if((pointer = strstr( tmp_bufferin, tmp_path)))
              {
		if(isgraph(*(pointer + strlen(tmp_path))))
		  break; /* there is something behind the path name, 
			    for instance like it would find /cdrom but 
			    also /cdrom1 since /cdrom is a subset of /cdrom1 */
		/*isblank should work too but how do you do that with 
		  the "gnu extension"? (man isblank) */

		while(isgraph( (int) tmp_bufferin[k] ))
	          {
		    device[k] = tmp_bufferin[k];
		    k++;
	          }
		if(isdigit(device[k-1]))
	          {
	              fprintf(stderr, "Hmm, the last char in the path that gets mounted to %s is a number, thats not correct\n", path);
	              fprintf(stderr, "You should consider changing it in /etc/fstab\n");
	              k--;
	          }
		device[k] = '\0';
	      }
	    memset(tmp_bufferin, 0, MAX_STRING * sizeof(char));
          }
          fclose(tmp_streamin);
	  if(!strstr(device, "/dev"))
	    {
	      fprintf(stderr, "Could not find the provided path (%s), typo?\n",path);
	      device[0] = '\0';
	      return -1;
	    }
      }
    else
      {
	fprintf(stderr, "Could not read /etc/fstab!");
	device[0] = '\0';
	return -1;
      }
  }
  return mounted;
}


/* function to get the device WITHOUT a given pathname */
/* therefore called oyo - on your own */
/* returns 0 if successful and NOT mounted        */
/*         1 if successful and mounted            */
/* returns <0 if error                            */
int get_device_oyo( char *path, char *device )
{ //oyo
  FILE	*tmp_streamin;
  char	tmp_bufferin[MAX_STRING];
  char  tmp_path[20];
  int   k = 0, l = 0, cd_count = 0;
    
  /*
   *read the device out of /etc/fstab 
   */

 if( (tmp_streamin = fopen("/etc/mtab","r")))
   {
     strcpy(tmp_path, "iso9660");
     memset( tmp_bufferin, 0, MAX_STRING * sizeof(char));
     while( fgets( tmp_bufferin, MAX_STRING, tmp_streamin )) 
     {
       if(strstr( tmp_bufferin, tmp_path))
	 {
	   cd_count++; /*count every cd we find */
	   /*
	     extract the device
	   */

	   while(isgraph( (int) tmp_bufferin[k] ))
	     {
	       device[k] = tmp_bufferin[k];
	       k++;
	     }
	   if(isdigit(device[k-1]))
	     {
	       fprintf(stderr, "Hmm, the last char in the path that gets mounted to %s is a number, thats not correct\n", path);
	       fprintf(stderr, "You should consider changing it in /etc/fstab\n");
	       k--;
	     }
	   device[k] = '\0';

	   /*traverse the gap*/

	   if( isgraph( (int) tmp_bufferin[k] ))
	     k++;
	   while(!(isgraph( (int) tmp_bufferin[k] ))) 
	     k++;

	   /*
	     extract the path the device has been mounted to
	   */

	   while(isgraph( (int) tmp_bufferin[k] ))
	     {
	       path[l] = tmp_bufferin[k];
	       k++;
	       l++;
	     }
	   path[l] = '\0';

	 }
       memset(tmp_bufferin, 0, MAX_STRING * sizeof(char));
       k = 0, l = 0; /*for the next run
		       we take the last entry in /etc/mtab since that has been 
		       mounted the last and we assume that this is the
		       dvd. 
		     */
     }
     fclose(tmp_streamin);

     if(cd_count == 0)
       { /* no cd found? Then user should mount it */
	 fprintf(stderr, "There seems to be no cd/dvd mounted. Please do that..\n");
	 return -1;
       }
   }
 else
   {
     fprintf(stderr, "Could not read /etc/mtab!");
     return -1;
   }
 return cd_count;
}



/******************* get the whole vob size via stat() ******************/

off_t get_vobsize(int title, char *provided_input_dir) 
{
      char  path_to_vobs[278], path_to_vobs1[278], path_to_vobs2[278], path_to_vobs3[278];
      char  temp[278], temp1[3], stat_path[278];
      int   subvob;
      FILE  *tmp_streamin1;
      struct stat buf;
      off_t  vob_size = 0;     

      sprintf(temp,"%d", title); /*which title-vob */
      
      /* first option: path/vts_xx_yy.vob */
      strcpy( path_to_vobs, provided_input_dir);
      strcat( path_to_vobs, "/vts_");
      if (title < 10)
	{
	  strcat( path_to_vobs, "0");
	  strcat( path_to_vobs, temp);
	}
      else
	{
	  strcat( path_to_vobs, temp);
	}
      
      /* second option: path/VTS_XX_YY.VOB */
      strcpy( path_to_vobs1, provided_input_dir);
      strcat( path_to_vobs1, "/VTS_");
      if (title < 10)
	{
	  strcat( path_to_vobs1, "0");
	  strcat( path_to_vobs1, temp);
	}
      else
	{
	  strcat( path_to_vobs1, temp);
	}
      
      /* third option: path/VIDEO_TS/VTS_XX_YY.VOB */
      strcpy( path_to_vobs2, provided_input_dir);
      strcat( path_to_vobs2, "/VIDEO_TS/VTS_");
      if (title < 10)
	{
	  strcat( path_to_vobs2, "0");
	  strcat( path_to_vobs2, temp);
	}
      else
	{
	  strcat( path_to_vobs2, temp);
	}

      /* fourth option: path/video_ts/vts_xx_yy.vob */
      strcpy( path_to_vobs3, provided_input_dir);   
      strcat( path_to_vobs3, "/video_ts/vts_");
      if (title < 10)
	{
	  strcat( path_to_vobs3, "0");
	  strcat( path_to_vobs3, temp);
	}
      else
	{
	  strcat( path_to_vobs3, temp);
	}

      /*
       * extract the size of the files on dvd using stat
       */
      strcpy(stat_path, path_to_vobs);
      strcat(stat_path, "_1.vob");
      if(( tmp_streamin1 = fopen( stat_path, "r")) != NULL) /*check if this path is correct*/
	{
	  fclose (tmp_streamin1);
	  subvob = 1;
	  while(!stat(stat_path, &buf))
	    {
	      /* adjust path for next subvob */
	      subvob++;
	      strcpy(stat_path, path_to_vobs);
	      strcat(stat_path, "_");
	      sprintf(temp1, "%d", subvob);
	      strcat( stat_path, temp1);
	      strcat(stat_path, ".vob");
	      
	      vob_size += buf.st_size;
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",vob_size); */
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",buf.st_size); */
	    }
	}
      
      strcpy(stat_path, path_to_vobs1);
      strcat(stat_path, "_1.VOB");
      if(( tmp_streamin1 = fopen( stat_path, "r")) != NULL) /*check if this path is correct */
	{
	  fclose (tmp_streamin1);
	  subvob = 1;
	  while(!stat(stat_path, &buf))
	    {
	      /* adjust path for next subvob */
	      subvob++;
	      strcpy(stat_path, path_to_vobs1);
	      strcat(stat_path, "_");
	      printf(temp1, "%d", subvob);
	      strcat( stat_path, temp1);
	      strcat(stat_path, ".VOB");
	      
	      vob_size += buf.st_size;
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",vob_size); */
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",buf.st_size); */
	    }
	}
      
      strcpy(stat_path, path_to_vobs2);
      strcat(stat_path, "_1.VOB");
      if(( tmp_streamin1 = fopen( stat_path, "r")) != NULL) /*check if this path is correct */
	{
	  fclose ( tmp_streamin1 );
	  subvob = 1;
	  while(!stat(stat_path, &buf))
	    {
	      /* adjust path for next subvob */
	      subvob++;
	      strcpy(stat_path, path_to_vobs2);
	      strcat(stat_path, "_");
	      sprintf(temp1, "%d", subvob);
	      strcat( stat_path, temp1);
	      strcat(stat_path, ".VOB");
	     
	      vob_size += buf.st_size;
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",vob_size); */
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",buf.st_size); */
	    }
	}
      strcpy(stat_path, path_to_vobs3);
      strcat(stat_path, "_1.vob");
      if(( tmp_streamin1 = fopen( stat_path, "r")) != NULL) /*check if this path is correct */
	{
	  fclose (tmp_streamin1);
	  subvob = 1;
	  while(!stat(stat_path, &buf))
	    {
	      /* adjust path for next subvob */
	      subvob++;
	      strcpy(stat_path, path_to_vobs3);
	      strcat(stat_path, "_");
	      sprintf(temp1, "%d", subvob);
	      strcat( stat_path, temp1);
	      strcat(stat_path, ".vob");
	      
	      vob_size += buf.st_size;
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",vob_size); */
/* 	      fprintf(stderr,"debug: vob_size: %lli\n",buf.st_size); */
	    }
	}

      return (off_t) vob_size; /* think that (off_t) is not really needed here?
				  as it is defined as off_t and the function is
				  also defined as off_t */
}










